#include <cstring>
#include <fstream>
#include <math.h>

#include "aubo_sdk/rtde.h"
#include "aubo_sdk/rpc.h"
#include "aubo_sdk/script.h"

using namespace arcs::aubo_sdk;
using namespace arcs::common_interface;

#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif

double distance(const std::vector<double> &a, const std::vector<double> &b)
{
    double res = 0.;
    if (a.size() != b.size()) {
        return -1;
    }

    for (int i = 0; i < (int)a.size(); i++) {
        res += (a[i] - b[i]) * (a[i] - b[i]);
    }
    return sqrt(res);
}

void waitArrivel(RobotInterfacePtr impl, std::vector<double> target)
{
    while (1) {
        auto cur = impl->getRobotState()->getJointPositions();
        double dis = distance(cur, target);
        if (dis < 0.0001) {
            break;
        }
        printf("cur:%f,%f,%f,%f,%f,%f\n", cur.at(0), cur.at(1), cur.at(2),
               cur.at(3), cur.at(4), cur.at(5));
        printf("target:%f,%f,%f,%f,%f,%f\n", target.at(0), target.at(1),
               target.at(2), target.at(3), target.at(4), target.at(5));
        std::this_thread::sleep_for(std::chrono::milliseconds(1));
    }
}

template <typename T>
inline std::ostream &operator<<(std::ostream &os, const std::vector<T> &list)
{
    for (size_t i = 0; i < list.size(); i++) {
        os << list.at(i);
        if (i != (list.size() - 1)) {
            os << ",";
        }
    }
    return os;
}

class TrajectoryIo
{
public:
    TrajectoryIo(const char *filename)
    {
        input_file_.open(filename, std::ios::in);
    }
    ~TrajectoryIo() { input_file_.close(); }

    std::vector<std::vector<double>> parse()
    {
        std::vector<std::vector<double>> res;
        std::string tmp;
        int linenum = 1;
        while (std::getline(input_file_, tmp, '\n')) {
            try {
                auto q = split(tmp, ",");
                res.push_back(q);
            } catch (const char *p) {
                std::cerr << "Line: " << linenum << " \"" << p << "\""
                          << " is not a number of double" << std::endl;
                break;
            }
            linenum++;
        }
        return res;
    }
    std::vector<double> split(const std::string &str, const char *delim)
    {
        std::vector<double> res;
        if ("" == str) {
            return res;
        }
        //先将要切割的字符串从string类型转换为char*类型
        char *strs = new char[str.length() + 1]; //不要忘了
        std::strcpy(strs, str.c_str());

        char *p = std::strtok(strs, delim);
        char *endp = nullptr;
        while (p) {
            double v = std::strtod(p, &endp);
            if (endp[0] != 0 && endp[0] != '\r') {
                delete[] strs;
                strs = nullptr;
                throw p;
            }
            res.push_back(v); //存入结果数组
            p = std::strtok(nullptr, delim);
        }

        if (strs) {
            delete[] strs;
            strs = nullptr;
        }

        return res;
    }

private:
    std::ifstream input_file_;
};

const char *SERVOJ_SCRIPT = R"(local aubo = require('aubo')
    local sched = sched or aubo.sched
    local math = aubo.math or math
    local sync = sched.sync
return function(api)
    local names = api:getRobotNames()
    robot = api:getRobotInterface(names[1])
    local function servoj(...)
        return robot:getMotionControl():servoJoint(...)
    end
    local function get_target()
        local target_q = robot:getRobotState():getJointPositions()
        for i=1,6 do
            target_q[i] = api:getRegisterControl():getDoubleInput(i-1)
        end
        return target_q
    end

    local running = true
    api:getRegisterControl():setInt32Input(0, 1)
    while running do
        local flag = api:getRegisterControl():getInt32Input(0)
        if flag == 6 then
            -- textmsg("servoj: ")
            servoj(get_target(), 0.1, 0.2, 0.005, 0.1, 200)
        elseif flag == -1 then
            running = false
        end
        sync()
    end
    textmsg("running = false")
end


)";

int exampleServoj(RpcClientPtr rpc, RtdeClientPtr rtde, ScriptClientPtr script)
{
    int chanel_out =
        rtde->setTopic(true,
                       { "input_int_registers_0", "input_double_registers_0",
                         "input_double_registers_1", "input_double_registers_2",
                         "input_double_registers_3", "input_double_registers_4",
                         "input_double_registers_5" },
                       200, 0);
    int chanel_in = rtde->setTopic(
        false,
        { "R1_actual_q", "R1_actual_current", "R1_robot_mode", "R1_safety_mode",
          "runtime_state", "R1_actual_TCP_pose" },
        50, 0);

    // 实时状态获取
    std::vector<double> actual_q_{ std::vector<double>(6, 0.) };
    std::vector<double> actual_current_{ std::vector<double>(6, 0.) };
    std::vector<double> actual_TCP_pose_{ std::vector<double>(6, 0.) };

    std::vector<double> joint_temperatures_{ std::vector<double>(6, 0.) };
    RobotModeType robot_mode_{ RobotModeType::NoController };
    SafetyModeType safety_mode_{ SafetyModeType::Normal };
    RuntimeState runtime_state_{ RuntimeState::Stopped };

    rtde->subscribe(chanel_in, [&](InputParser &parser) {
        actual_q_ = parser.popVectorDouble();
        actual_current_ = parser.popVectorDouble();
        robot_mode_ = parser.popRobotModeType();
        safety_mode_ = parser.popSafetyModeType();
        runtime_state_ = parser.popRuntimeState();
        actual_TCP_pose_ = parser.popVectorDouble();

        // std::cout << actual_q_ << std::endl;
    });

    TrajectoryIo input("../trajs/record6.offt");
    auto traj = input.parse();
    auto traj_sz = traj.size();
    if (traj_sz == 0) {
        // 没有轨迹
        std::cerr << "No waypoints" << std::endl;
        return 0;
    }
    std::vector<double> q1 = traj[0];

    // 移动到第一个点
    printf("goto p1\n");

    auto robot_name = rpc->getRobotNames().front();

    rpc->getRuntimeMachine()->start();
    rpc->getRobotInterface(robot_name)
        ->getMotionControl()
        ->moveJoint(q1, M_PI, M_PI, 0., 0.);
    waitArrivel(rpc->getRobotInterface(robot_name), q1);

    script->sendString(SERVOJ_SCRIPT);

    size_t i = 1;
    while (1) {
        rtde->publish(chanel_out, [&](OutputBuilder &ob) {
            ob.push(6);
            ob.push(traj[i][0]);
            ob.push(traj[i][1]);
            ob.push(traj[i][2]);
            ob.push(traj[i][3]);
            ob.push(traj[i][4]);
            ob.push(traj[i][5]);
            std::cout << i << ": " << traj[i] << std::endl;
            i++;
        });

        if (i == traj_sz) {
            rtde->publish(chanel_out, [&](OutputBuilder &ob) {
                std::cout << i << ": end" << std::endl;
                ob.push(-1);
                ob.push(0);
                ob.push(0);
                ob.push(0);
                ob.push(0);
                ob.push(0);
                ob.push(0);
                i++;
            });
            break;
        }

        std::this_thread::sleep_for(std::chrono::milliseconds(5));
    }

    script->disconnect();
    rtde->disconnect();
    rpc->disconnect();

    std::cout << "end servoj " << std::endl;
    return 0;
}

#define ip_local "127.0.0.1"

int main(int argc, char **argv)
{
    auto rpc_cli = std::make_shared<RpcClient>();
    rpc_cli->connect(ip_local, 30004);
    rpc_cli->login("aubo", "123456");

    auto rtde_cli = std::make_shared<RtdeClient>();
    rtde_cli->connect(ip_local, 30010);
    rtde_cli->login("aubo", "123456");

    auto script_cli = std::make_shared<ScriptClient>();
    script_cli->connect(ip_local, 30002);
    script_cli->login("aubo", "123456");

    exampleServoj(rpc_cli, rtde_cli, script_cli);
}
