#ifndef AUBO_SDK_AXES_INTERFACE_H
#define AUBO_SDK_AXES_INTERFACE_H

#include <aubo/sync_move.h>
#include <aubo/trace.h>

namespace arcs {
namespace common_interface {

enum class AxesModeType : int
{
    Disconnected,   ///< 未连接
    PowerOff,       ///< 断电
    BrakeReleasing, ///< 刹车松开中
    Idle,           ///< 空闲
    Running,        ///< 运行中
    Fault           ///< 错误状态
};

class AxesControl
{
public:
    virtual ~AxesControl() = default;

    /// 通电
    virtual int poweronExt() = 0;

    /// 断电
    virtual int poweroffExt() = 0;

    /// 使能
    virtual int enableExt() = 0;

    /**
     * 设置外部轴的安装位姿(相对于世界坐标系)
     *
     * @param pose
     * @return
     */
    virtual int setExtMountingPose(const std::vector<double> &pose) = 0;

    /// 运动到指定点
    /// 旋转或者平移
    virtual int moveExtJoint(double pos, double v, double a, double duration,
                             double blend_radius) = 0;

    /// 制定目标运动速度
    virtual int speedExtJoint(double v, double a, double duration,
                              double blend_radius) = 0;
};
using AxesControlPtr = std::shared_ptr<AxesControl>;

class AxesState
{
public:
    virtual ~AxesState() = default;

    /// 获取当前外部轴的状态
    virtual AxesModeType getAxesModeType() = 0;

    /// 获取外部轴安装位姿
    virtual std::vector<double> getExtMountingPose() = 0;

    /**
     * 获取相对于安装坐标系的位姿，外部轴可能为变位机或者导轨
     */
    virtual std::vector<double> getExtPose() = 0;

    /// 获取外部轴位置
    virtual double getExtPosition() = 0;
    virtual double getExtVelocity() = 0;
    virtual double getExtAcceleration() = 0;
    virtual double getExtCurrent() = 0;

    virtual double getExtTemperature() = 0;
    virtual double getExtBusVoltage() = 0;
    virtual double getExtBusCurrent() = 0;

    /// 获取外部轴最大位置
    virtual double getExtMaxPosition() = 0;
    virtual double getExtMinPosition() = 0;
    virtual double getExtMaxVelocity() = 0;
    virtual double getExtMaxAcceleration() = 0;
};
using AxesStatePtr = std::shared_ptr<AxesState>;

/**
 * 外部轴API接口
 */
class AxesInterface
{
public:
    virtual ~AxesInterface() = default;

    /// 获取同步运动接口
    virtual SyncMovePtr getSyncMove() = 0;

    /// 获取告警信息接口
    virtual TracePtr getTrace() = 0;

    virtual AxesControlPtr getAxesControl() = 0;
    virtual AxesStatePtr getAxesState() = 0;
};
using AxesInterfacePtr = std::shared_ptr<AxesInterface>;

} // namespace common_interface
} // namespace arcs

#endif // AUBO_SDK_ROBOT_INTERFACE_H
