#ifndef AUBO_SDK_ROBOT_ALGORITHM_INTERFACE_H
#define AUBO_SDK_ROBOT_ALGORITHM_INTERFACE_H

#include <string>
#include <vector>
#include <memory>
#include <functional>

#include <aubo/global_config.h>
#include <aubo/type_def.h>

namespace arcs {
namespace common_interface {

/**
 * 机器人算法相关的对外接口
 */
class ARCS_ABI_EXPORT RobotAlgorithm
{
public:
    RobotAlgorithm();
    virtual ~RobotAlgorithm();

    /**
     * 力传感器标定算法
     */
    ForceSensorCalibResult calibrateTcpForceSensor(
        const std::vector<std::vector<double>> &force,
        const std::vector<std::vector<double>> &q);

    /**
     * 负载辨识算法接口
     * 输入关节角度和(工具端)力矩传感器读数，输出质量，质心偏移等等
     * q: 关节角度集合
     * forces: 对应的末端力/力矩集合
     * result: 辨识的结果
     */
    ForceSensorCalibResult payloadIdentify(
        const std::vector<std::vector<double>> &q,
        const std::vector<std::vector<double>> &forces);

    /**
     * 关节摩擦力模型辨识算法接口
     */
    bool frictionModelIdentify(const std::vector<std::vector<double>> &q,
                               const std::vector<std::vector<double>> &qd,
                               const std::vector<std::vector<double>> &qdd,
                               const std::vector<std::vector<double>> &temp);

    // clang-format off
    // 工具坐标系标定算法接口
    // https://www.cnblogs.com/beta-1999/p/13121564.html
    // 4点法: 示教四个位置P1 P2 P3 P4，每个位置的工具的Z轴角度间隔在45-90之间且不在同一个平面，最终TCP为(X,Y,Z,0,0,0)
    // P5点沿工具的+X方向移动最好移动100mm以上。
    // P6点沿工具的+Z方向移动最好移动100mm以上。(工具升降的方向)
    // 保持P4、P5、P6的姿态不变，P5以P4为基点沿着作业平台的+X方向移动，P6以P4为基点沿着作业平台的+Z方向移动
    // 输入多组关节角度和标定类型，输出工具坐标系偏移值
    // clang-format on
    ResultWithErrno tcpOffsetIdentify(
        const std::vector<std::vector<double>> &tool_poses);

    ResultWithErrno coordinateCalibrate(
        int type, const std::vector<std::vector<double>> &poses);

    // 手眼标定算法接口
    // https://blog.csdn.net/yaked/article/details/77161160
    // 结果为相机相对于机器人基坐标系的位姿
    ResultWithErrno eyeTohandCalibration(
        const std::vector<std::vector<double>> &hand,
        const std::vector<std::vector<double>> &eye);

    // 手眼标定算法接口
    ResultWithErrno eyeInhandCalibration(
        const std::vector<std::vector<double>> &hand,
        const std::vector<std::vector<double>> &eye);

    // 工件坐标系标定算法接口(需要在调用之前正确的设置机器人的TCP偏移)
    // 输入多组关节角度和标定类型，输出工件坐标系位姿(相对于机器人基坐标系)
    ResultWithErrno calibWorkpieceCoordinatePara(
        const std::vector<std::vector<double>> &q, int type);

    // 动力学正解
    ResultWithErrno forwardDynamics(const std::vector<double> &q,
                                    const std::vector<double> &torqs);

    // 运动学正解
    // 输入关节角度，输出TCP位姿
    ResultWithErrno forwardKinematics(const std::vector<double> &q);

    // 运动学逆解
    // 输入TCP位姿和参考关节角度，输出关节角度
    // 返回值为计算结果以及错误代码
    ResultWithErrno inverseKinematics(const std::vector<double> &qnear,
                                      const std::vector<double> &pose);

    // 姿态变换
    // F_B_A = F_B_C * F_C_A
    std::vector<double> poseTrans(const std::vector<double> &pose_from,
                                  const std::vector<double> &pose_from_to);

    std::vector<double> poseInverse(const std::vector<double> &pose);

    // 姿态变换
    // F_B_C = F_B_A * inv(F_C_A)
    std::vector<double> poseTransInv(const std::vector<double> &F_B_A,
                                     const std::vector<double> &F_C_A);

    // 姿态旋转
    std::vector<double> poseRotation(const std::vector<double> &pose,
                                     const std::vector<double> &rotv);

    // 四元数与欧拉角互相转换
    std::vector<double> rpyToQuaternion(const std::vector<double> &rpy);

    std::vector<double> quaternionToRpy(const std::vector<double> &quat);

    /**
     * 求解movej之间的轨迹点
     * q1: movej的起点
     * r1: 在q1处的交融半径
     * q2: movej的终点
     * r2: 在q2处的交融半径
     * d: 采样距离
     * 返回值: q1~q2 之间不包含交融段的笛卡尔空间离散轨迹点(x,y,z)集合
     */
    std::vector<std::vector<double>> pathMovej(const std::vector<double> &q1,
                                               double r1,
                                               const std::vector<double> &q2,
                                               double r2, double d);

    /**
     * 求解交融的轨迹点
     * type
     * 0-movej and movej
     * 1-movej and movel
     * 2-movel and movej
     * 2-movel and movel
     * q_start: 交融前路径的起点
     * q_via: 在q1处的交融半径
     * q_to: 交融后路径的终点
     * r: 在q_via处的交融半径
     * d: 采样距离
     * 返回值: q_via处的交融段笛卡尔空间离散轨迹点(x,y,z)集合
     */
    std::vector<std::vector<double>> pathBlend3Points(
        int type, const std::vector<double> &q_start,
        const std::vector<double> &q_via, const std::vector<double> &q_to,
        double r, double d);

protected:
    void *d_;
};
using RobotAlgorithmPtr = std::shared_ptr<RobotAlgorithm>;

// clang-format off
#define ROBOT_ALGORITHM_DECLARES                                   \
    X(RobotAlgorithm, 2, calibrateTcpForceSensor, force, q)        \
    X(RobotAlgorithm, 2, payloadIdentify, q, forces)               \
    X(RobotAlgorithm, 4, frictionModelIdentify, q, qd, qdd, temp)  \
    X(RobotAlgorithm, 1, tcpOffsetIdentify, tool_poses)            \
    X(RobotAlgorithm, 2, coordinateCalibrate, type, poses)         \
    X(RobotAlgorithm, 2, eyeTohandCalibration, hand, eye)          \
    X(RobotAlgorithm, 2, eyeInhandCalibration, hand, eye)          \
    X(RobotAlgorithm, 2, calibWorkpieceCoordinatePara, q, type)    \
    X(RobotAlgorithm, 2, forwardDynamics, q, torqs)                \
    X(RobotAlgorithm, 1, forwardKinematics, q)                     \
    X(RobotAlgorithm, 2, inverseKinematics, qnear, pose)           \
    X(RobotAlgorithm, 2, poseTrans, pose_from, pose_from_to)       \
    X(RobotAlgorithm, 1, poseInverse, pose)                        \
    X(RobotAlgorithm, 2, poseTransInv, F_B_A, F_C_A)               \
    X(RobotAlgorithm, 2, poseRotation, pose, rotv)                 \
    X(RobotAlgorithm, 1, rpyToQuaternion, rpy)                     \
    X(RobotAlgorithm, 1, quaternionToRpy, quant)                   \
    X(RobotAlgorithm, 5, pathMovej, q1, r1, q2, r2, override)      \
    X(RobotAlgorithm, 6, pathBlend3Points, type, q_start, q_via, q_to, r, d)

// clang-format on

} // namespace common_interface
} // namespace arcs

#endif // AUBO_SDK_ROBOT_ALGORITHM_INTERFACE_H
