#ifndef AUBO_SDK_ROBOT_STATE_INTERFACE_H
#define AUBO_SDK_ROBOT_STATE_INTERFACE_H

#include <vector>

#include <aubo/global_config.h>
#include <aubo/type_def.h>

namespace arcs {
namespace common_interface {

class ARCS_ABI_EXPORT RobotState
{
public:
    RobotState();
    virtual ~RobotState();

    /**
     * 获取机器人的模式状态
     */
    RobotModeType getRobotModeType();

    /**
     * 获取安全模式
     */
    SafetyModeType getSafetyModeType();

    /**
     * 机器人是否已经停止下来
     */
    bool isSteady();

    /**
     * 机器人是否已经在安全限制之内
     */
    bool isWithinSafetyLimits();

    /**
     * 获取TCP的位姿
     */
    std::vector<double> getTcpPose();

    /**
     * 获取当前目标位姿
     */
    std::vector<double> getTargetTcpPose();

    /**
     * 获取工具端的位姿（不带TCP偏移）
     */
    std::vector<double> getToolPose();

    /**
     * 获取TCP速度
     */
    std::vector<double> getTcpSpeed();

    /**
     * 获取TCP的力/力矩
     */
    std::vector<double> getTcpForce();

    /**
     * 获取肘部的位置
     */
    std::vector<double> getElbowPosistion();

    /**
     * 获取肘部速度
     */
    std::vector<double> getElbowVelocity();

    /**
     * 获取基座力/力矩
     */
    std::vector<double> getBaseForce();

    /**
     * 获取TCP目标位姿
     */
    std::vector<double> getTcpTargetPose();

    /**
     * 获取TCP目标速度
     */
    std::vector<double> getTcpTargetSpeed();

    /**
     * 获取TCP目标力/力矩
     */
    std::vector<double> getTcpTargetForce();

    /**
     * 获取机械臂关节标志接口
     */
    std::vector<JointStateType> getJointState();

    /**
     * 获取关节的伺服状态
     */
    std::vector<JointServoModeType> getJointServoMode();

    /**
     * 获取机械臂关节角度接口
     */
    std::vector<double> getJointPositions();

    /**
     * 获取机械臂关节速度接口
     */
    std::vector<double> getJointSpeeds();

    /**
     * 获取机械臂关节加速度接口
     */
    std::vector<double> getJointAccelerations();

    /**
     * 获取机械臂关节力矩接口
     */
    std::vector<double> getJointTorqueSensors();

    /**
     * 获取底座力传感器读数
     */
    std::vector<double> getBaseForceSensor();

    /**
     * 获取TCP力传感器读数
     */
    std::vector<double> getTcpForceSensors();

    /**
     * 获取机械臂关节电流接口
     */
    std::vector<double> getJointCurrents();

    /**
     * 获取机械臂关节电压接口
     */
    std::vector<double> getJointVoltages();

    /**
     * 获取机械臂关节温度接口
     */
    std::vector<double> getJointTemperatures();

    /**
     * 获取关节 UniqueId
     */
    std::vector<std::string> getJointUniqueIds();

    /**
     * 获取关节固件版本
     */
    std::vector<int> getJointFirmwareVersions();

    /**
     * 获取关节硬件版本
     */
    std::vector<int> getJointHardwareVersions();

    std::string getMasterBoardUniqueId();

    /**
     * 获取MasterBoard固件版本
     */
    int getMasterBoardFirmwareVersion();

    /**
     * 获取MasterBoard硬件版本
     */
    int getMasterBoardHardwareVersion();

    std::string getSlaveBoardUniqueId();

    /**
     * 获取SlaveBoard固件版本
     */
    int getSlaveBoardFirmwareVersion();

    /**
     * 获取SlaveBoard硬件版本
     */
    int getSlaveBoardHardwareVersion();

    /**
     * 获取工具端全球唯一ID
     */
    std::string getToolUniqueId();

    /**
     * 获取工具端固件版本
     */
    int getToolFirmwareVersion();

    /**
     * 获取工具端硬件版本
     */
    int getToolHardwareVersion();

    /**
     * 获取底座全球唯一ID
     */
    std::string getPedestalUniqueId();

    /**
     * 获取底座固件版本
     */
    int getPedestalFirmwareVersion();

    /**
     * 获取底座硬件版本
     */
    int getPedestalHardwareVersion();

    /**
     * 获取机械臂关节目标位置角度接口
     */
    std::vector<double> getJointTargetPositions();

    /**
     * 获取机械臂关节目标速度
     */
    std::vector<double> getJointTargetSpeeds();

    /**
     * 获取机械臂关节目标加速度
     */
    std::vector<double> getJointTargetAccelerations();

    /**
     * 获取机械臂关节目标力矩
     */
    std::vector<double> getJointTargetTorques();

    /**
     * 获取机械臂关节目标电流
     */
    std::vector<double> getJointTargetCurrents();

    /**
     * 获取控制柜温度
     */
    double getControlBoxTemperature();

    /**
     * 获取母线电压
     */
    double getMainVoltage();

    /**
     * 获取母线电流
     */
    double getMainCurrent();

    /**
     * 获取机器人电压
     */
    double getRobotVoltage();

    /**
     * 获取机器人电流
     */
    double getRobotCurrent();

protected:
    void *d_;
};
using RobotStatePtr = std::shared_ptr<RobotState>;

// clang-format off
#define ROBOT_STATE_DECLARES                         \
    X(RobotState, 0, getRobotModeType)               \
    X(RobotState, 0, getSafetyModeType)              \
    X(RobotState, 0, isSteady)                       \
    X(RobotState, 0, isWithinSafetyLimits)           \
    X(RobotState, 0, getTcpPose)                     \
    X(RobotState, 0, getTargetTcpPose)               \
    X(RobotState, 0, getToolPose)                    \
    X(RobotState, 0, getTcpSpeed)                    \
    X(RobotState, 0, getTcpForce)                    \
    X(RobotState, 0, getElbowPosistion)              \
    X(RobotState, 0, getElbowVelocity)               \
    X(RobotState, 0, getBaseForce)                   \
    X(RobotState, 0, getTcpTargetPose)               \
    X(RobotState, 0, getTcpTargetSpeed)              \
    X(RobotState, 0, getTcpTargetForce)              \
    X(RobotState, 0, getJointState)                  \
    X(RobotState, 0, getJointServoMode)              \
    X(RobotState, 0, getJointPositions)              \
    X(RobotState, 0, getJointSpeeds)                 \
    X(RobotState, 0, getJointAccelerations)          \
    X(RobotState, 0, getJointTorqueSensors)          \
    X(RobotState, 0, getBaseForceSensor)             \
    X(RobotState, 0, getTcpForceSensors)             \
    X(RobotState, 0, getJointCurrents)               \
    X(RobotState, 0, getJointVoltages)               \
    X(RobotState, 0, getJointTemperatures)           \
    X(RobotState, 0, getJointUniqueIds)              \
    X(RobotState, 0, getJointFirmwareVersions)       \
    X(RobotState, 0, getJointHardwareVersions)       \
    X(RobotState, 0, getMasterBoardUniqueId)         \
    X(RobotState, 0, getMasterBoardFirmwareVersion)  \
    X(RobotState, 0, getMasterBoardHardwareVersion)  \
    X(RobotState, 0, getSlaveBoardUniqueId)          \
    X(RobotState, 0, getSlaveBoardFirmwareVersion)   \
    X(RobotState, 0, getSlaveBoardHardwareVersion)   \
    X(RobotState, 0, getToolUniqueId)                \
    X(RobotState, 0, getToolFirmwareVersion)         \
    X(RobotState, 0, getToolHardwareVersion)         \
    X(RobotState, 0, getPedestalUniqueId)            \
    X(RobotState, 0, getPedestalFirmwareVersion)     \
    X(RobotState, 0, getPedestalHardwareVersion)     \
    X(RobotState, 0, getJointTargetPositions)        \
    X(RobotState, 0, getJointTargetSpeeds)           \
    X(RobotState, 0, getJointTargetAccelerations)    \
    X(RobotState, 0, getJointTargetTorques)          \
    X(RobotState, 0, getJointTargetCurrents)         \
    X(RobotState, 0, getControlBoxTemperature)       \
    X(RobotState, 0, getMainVoltage)                 \
    X(RobotState, 0, getMainCurrent)                 \
    X(RobotState, 0, getRobotVoltage)                \
    X(RobotState, 0, getRobotCurrent)
// clang-format on
} // namespace common_interface
} // namespace arcs
#endif // AUBO_SDK_ROBOT_STATE_INTERFACE_H
